'use strict';
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const identity_1 = require("@azure/identity");
const storage_blob_1 = require("@azure/storage-blob");
const resetable_1 = __importDefault(require("resetable"));
class AzureStorage {
    constructor(config) {
        this.config = config;
        if (typeof config.connection_string !== 'undefined') {
            // eslint-disable-next-line
            this.AzureClient = storage_blob_1.BlobServiceClient.fromConnectionString(config.connection_string);
        }
        else {
            let credential;
            if (config.azure_tenant_id &&
                config.azure_client_id &&
                config.azure_client_secret) {
                credential = new identity_1.DefaultAzureCredential();
            }
            else if (config.name && config.key) {
                credential = new storage_blob_1.StorageSharedKeyCredential(config.name, config.key);
            }
            let url = `https://${config.name}.blob.core.windows.net`;
            if (typeof config.local_address !== 'undefined') {
                url = config.local_address;
            }
            const pipeline = (0, storage_blob_1.newPipeline)(credential);
            this.AzureClient = new storage_blob_1.BlobServiceClient(url, pipeline);
        }
        this._container = new resetable_1.default(config.container);
    }
    getBlockBlobClient(relativePath) {
        const container = this._container.pull();
        const containerClient = this.AzureClient.getContainerClient(container);
        return containerClient.getBlockBlobClient(relativePath);
    }
    container(container) {
        this._container.set(container);
        return this;
    }
    existsContainer(container, options = {}) {
        const containerClient = this.AzureClient.getContainerClient(container);
        return new Promise((resolve, reject) => {
            try {
                containerClient.exists(options).then(containerResult => {
                    resolve(containerResult);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    createContainer(container, options = {}) {
        const containerClient = this.AzureClient.getContainerClient(container);
        return new Promise((resolve, reject) => {
            try {
                containerClient.create(options).then(containerResult => {
                    this._container.set(containerResult);
                    resolve(containerResult);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    deleteContainer(container, options = {}) {
        const containerClient = this.AzureClient.getContainerClient(container);
        return new Promise((resolve, reject) => {
            try {
                containerClient.delete(options).then(containerResult => {
                    resolve(containerResult);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    /**
     * List all files within vitual folder if specified
     * If nothing is specified, root files within container will be listed
     * Disclaimer: Doesn't list virtual folders
     */
    async list(prefix = '', options = {}) {
        if (prefix !== null && prefix !== undefined && prefix !== '') {
            options.prefix = prefix;
        }
        const container = this._container.pull();
        const containerClient = this.AzureClient.getContainerClient(container);
        // const listBlobsResponse = await containerClient.listBlobsByHierarchy('/', options)
        const blobs = [];
        for await (const item of containerClient.listBlobsByHierarchy('/', options)) {
            if (item.kind !== 'prefix') {
                blobs.push(item);
            }
        }
        return blobs;
    }
    exists(relativePath, options = {}) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        return new Promise((resolve, reject) => {
            try {
                blockBlobClient.exists(options).then(exists => {
                    resolve(exists);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    put(relativePath, content, options = {}) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        return new Promise((resolve, reject) => {
            try {
                blockBlobClient.upload(content, content.length, options).then(response => {
                    resolve(response);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    putStream(relativePath, content) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        return new Promise((resolve, reject) => {
            try {
                blockBlobClient.uploadStream(content, content.length).then(response => {
                    resolve(response);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    delete(relativePath, options = {}) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        return new Promise((resolve, reject) => {
            try {
                blockBlobClient.delete(options).then(exists => {
                    resolve(exists);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    get(relativePath, options = {}) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        return new Promise((resolve, reject) => {
            try {
                blockBlobClient.downloadToBuffer(0, 0, options).then(file => {
                    resolve(file);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    getStream(relativePath, options = {}) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        return new Promise((resolve, reject) => {
            try {
                blockBlobClient.download(0, 0, options).then(file => {
                    resolve(file);
                });
            }
            catch (err) {
                reject(err);
            }
        });
    }
    async move(relativeSrcPath, relativeDestPath, options = {}) {
        const srcContainer = this._container.get();
        await this.copy(relativeSrcPath, relativeDestPath, options);
        return this.container(srcContainer).delete(relativeSrcPath);
    }
    async copy(relativeSrcPath, relativeDestPath, options = {}) {
        const container = this._container.pull();
        options.destContainer = options.destContainer || container;
        const srcBlockBlobClient = this.getBlockBlobClient(relativeSrcPath);
        const destBlockBlobClient = this.getBlockBlobClient(relativeDestPath);
        const url = await this.generateBlobSASURL(srcBlockBlobClient);
        try {
            return await destBlockBlobClient.syncCopyFromURL(url);
        }
        catch (err) {
            return err;
        }
    }
    async generateBlobSASURL(blockBlobClient, options = {}) {
        options.permissions = options.permissions || 'r';
        options.expiry = options.expiry || 3600;
        options.startsOn = options.startsOn || new Date();
        options.expiresOn =
            options.expiresOn ||
                new Date(options.startsOn.valueOf() + options.expiry * 1000);
        const blobSAS = await (0, storage_blob_1.generateBlobSASQueryParameters)({
            containerName: blockBlobClient.containerName,
            blobName: blockBlobClient.location,
            permissions: storage_blob_1.BlobSASPermissions.parse(options.permissions),
            startsOn: options.startsOn,
            expiresOn: options.expiresOn,
        }, blockBlobClient.credential);
        return `${blockBlobClient.url}?${blobSAS.toString()}`;
    }
    async getSignedUrl(relativePath, options = {}) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        const SASUrl = await this.generateBlobSASURL(blockBlobClient, options);
        return SASUrl;
    }
    getUrl(relativePath) {
        const blockBlobClient = this.getBlockBlobClient(relativePath);
        return unescape(blockBlobClient.url);
    }
}
exports.default = AzureStorage;
